<?php

namespace FiloBlu\Referrals\Plugin;

class UtilityPlugin
{

    protected $_referralHelper;

    public function __construct(
        \FiloBlu\Referrals\Helper\Core $referralHelper
    )
    {
        $this->_referralHelper = $referralHelper;
    }

    /*
     * This is needed because ONLY the user receiving a coupon from an invite can use THAT coupon
     * That coupon is not shareable!
     */
    public function afterCanProcessRule($utility, $result, $rule, $address){

        $result = $this->processRule($utility, $rule, $address);
        // The validation is already not true, no need to go forward!
        if(!$result){
            return $result;
        }

        // The module referral is not enabled, no need to go forward!
        if (!$this->_referralHelper->isEnabled()){
            //$this->_referralHelper->writeLog("FiloBlu\Referrals Module not enabled!");
            return $result;
        }

        $couponCode = $address->getQuote()->getCouponCode();
        /*
         * If more codes are used, I only check the last entered (the last in the list) since the previous ones have already been checked
         */
        $couponCodeArray = explode(',', $couponCode);
        if(count($couponCodeArray) > 1)
        {
            $couponCode = array_pop($couponCodeArray);
        }

        // The coupon is not a "invitational" coupon, no need to go forward!
        if (!$this->_referralHelper->isInvitationalTypeCoupon($couponCode)){
            return $result;
        }

        $customerId = $address->getQuote()->getCustomerId();

        // Remember, all coupons are unique!
        $invitation = $this->_referralHelper->getInvitationByCouponCode($couponCode);

        // There is no invitation with this coupon code! No need to go forward!
        if(!$invitation){
            $this->_referralHelper->writeLog("COUPON PLUGIN: Can't find invitation with coupon code {$couponCode}");
            return $result;
        }

        $this->_referralHelper->writeLog("COUPON PLUGIN: The customer_id {$customerId} is using the coupon {$couponCode}");

        // This customer $customerId, using this coupon $couponCode, is the sender of the invitation $invitation?
        $customerIsSender = false;
        $customerIsReceiver = false;
        if ($invitation->getCustomerId() == $customerId){
            $customerIsSender = true;
        }

        if (!$customerIsSender &&  $invitation->getReferralId() == $customerId) {
            $customerIsReceiver = true;
        }

        if(!$customerIsReceiver && !$customerIsSender){
            $this->_referralHelper->writeLog("COUPON PLUGIN: The customer_id {$customerId} is NOT the sender or the receiver of the invitation_id {$invitation->getInvitationId()}");
            return false;
        }

        //Now we have all we need to validate this coupon with the user
        $db_field = null;
        if($customerIsSender){
            $db_field = $this->_referralHelper->getCouponDbFieldNameForWhoSentTheInvite();
            $this->_referralHelper->writeLog("COUPON PLUGIN: The customer_id {$customerId} is the sender of the invitation_id {$invitation->getInvitationId()}");
        }
        if($customerIsReceiver){
            $db_field = $this->_referralHelper->getCouponDbFieldNameForWhoGotInvited();
            $this->_referralHelper->writeLog("COUPON PLUGIN: The customer_id {$customerId} is the receiver of the invitation_id {$invitation->getInvitationId()}");
        }

        $coupon_to_validate = $invitation->getData($db_field);

        // The coupon i'm trying to validate is not mine!!
        if ($couponCode != $coupon_to_validate){
            $this->_referralHelper->writeLog("COUPON PLUGIN: The coupon {$couponCode} does not belong to the customer_id {$customerId}");
            $rule->setIsValidForAddress($address, false);
            return false;
        }

        // All good!
        return $result;
    }


    /*
     * Rewrite canProcessRule method of class Magento\SalesRule\Model\Utility for multicode management
     */
    private function processRule($utility, $rule, $address){
        if ($rule->hasIsValidForAddress($address) && !$address->isObjectNew()) {
            return $rule->getIsValidForAddress($address);
        }

        /**
         * check per coupon usage limit
         */
        if ($rule->getCouponType() != \Magento\SalesRule\Model\Rule::COUPON_TYPE_NO_COUPON) {
            $couponCode = $address->getQuote()->getCouponCode();
            if (strlen($couponCode)) {
                /*
                 * If more codes are used, I only check the last entered (the last in the list) since the previous ones have already been checked
                 */
                $couponCodeArray = explode(',', $couponCode);
                if(count($couponCodeArray) > 1)
                {
                    $couponCode = array_pop($couponCodeArray);
                }
                /** @var \Magento\SalesRule\Model\Coupon $coupon */
                $coupon = $utility->couponFactory->create();
                $coupon->load($couponCode, 'code');
                if ($coupon->getId()) {
                    // check entire usage limit
                    if ($coupon->getUsageLimit() && $coupon->getTimesUsed() >= $coupon->getUsageLimit()) {
                        $rule->setIsValidForAddress($address, false);
                        return false;
                    }
                    // check per customer usage limit
                    $customerId = $address->getQuote()->getCustomerId();
                    if ($customerId && $coupon->getUsagePerCustomer()) {
                        $couponUsage = $utility->objectFactory->create();
                        $utility->usageFactory->create()->loadByCustomerCoupon(
                            $couponUsage,
                            $customerId,
                            $coupon->getId()
                        );
                        if ($couponUsage->getCouponId() &&
                            $couponUsage->getTimesUsed() >= $coupon->getUsagePerCustomer()
                        ) {
                            $rule->setIsValidForAddress($address, false);
                            return false;
                        }
                    }
                }
            }
        }

        /**
         * check per rule usage limit
         */
        $ruleId = $rule->getId();
        if ($ruleId && $rule->getUsesPerCustomer()) {
            $customerId = $address->getQuote()->getCustomerId();
            /** @var \Magento\SalesRule\Model\Rule\Customer $ruleCustomer */
            $ruleCustomer = $utility->customerFactory->create();
            $ruleCustomer->loadByCustomerRule($customerId, $ruleId);
            if ($ruleCustomer->getId()) {
                if ($ruleCustomer->getTimesUsed() >= $rule->getUsesPerCustomer()) {
                    $rule->setIsValidForAddress($address, false);
                    return false;
                }
            }
        }
        $rule->afterLoad();
        /**
         * quote does not meet rule's conditions
         */
        if (!$rule->validate($address)) {
            $rule->setIsValidForAddress($address, false);
            return false;
        }
        /**
         * passed all validations, remember to be valid
         */
        $rule->setIsValidForAddress($address, true);
        return true;
    }

}